/*---------------------------------------------------------------------------*/
/* Bits & Bytes Parallel Port support in Turbo C */
/* (c) 1998 Mountain ClockWorks / Terry King (noncommercial use permitted)   */
/*---------------------------------------------------------------------------*/

/*------------------( #include LIBRARIES )-------------------------*/
#include <stdio.h>
#include <dos.h>
/*------------------( CONSTANTS )----------------------------------*/

/*-------( IBM type Parallel Printer Port addresses )------------------------*/
#define LPT1            0x0378; // {PARALLEL card base address }
#define LPT2            0x0278; // {PARALLEL card base address }
#define LPTM            0x03BC; // {PARALLEL card base address }
#define DATALATCH       0x0000; // {ADDER get to DO "DATA LATCH" register }
#define PRINTERSTATUS   0x0001; // {ADDER get to DI "PRINTER STATUS" register }
#define PRINTERCONTROL  0x0002; // {ADDER get to DO "PRINTER CONTROL" register }

/*-----------( DATA Constants )-----------------------------------*/

#define RESETDATA          0x00; // {Value to set DIDO to at Reset Time }
#define DONORMAL           0x00; // {Output bits for Normal + TRUE DO}
#define SENSEBITSDEFAULT   0xFF; // {Sensor bits to Invert for easy understanding}
#define DIINVERT           0x92; // {Input bits to fix inversions }


/*-- ParPort CONSTANTS --*/
#define MAXBITNUM 7

/*-- ParPort ERROR NUMBERS --*/
#define ERR_BADBITNUM  -1            //Bit Number not 0 .. 7

/*------------------( VARIABLES )----------------------------------*/
unsigned char  BitsNowOn   ;
unsigned char  SenseBitsNow;
unsigned char  SenseBitsInvert;
int            ParPortAddr ; // Parallel port Base address
int            PrtStatAddr ; // Parallel Port Status address
int            PrtCtrlAddr ; // Parallel Port Control address


/*------------------( FUNCTION PROTOTYPES )--------------------------------*/
int ParPortInit();
void ParPortOut(unsigned char OutData);
unsigned char ParPortIn();
int BitOff(unsigned char BitNum);
int BitOn(unsigned char BitNum);


/*------------------( MAIN PROGRAM: Temporary for Test )-----------*/
 int main()
{
unsigned char TestData;

 printf("ParPort in 'C' ?   Version 5.1\n");
 ParPortInit();
 ParPortOut(0x55);
 delay(2000);
 ParPortOut(0xAA);
 delay(2000);
 ParPortOut(0x00);
 delay(2000);
 BitOn(4);
 delay(1000);
 BitOn(0);
 delay(2000);
 BitOff(4);
 delay(1000);
 BitOff(0);
 printf("Parallel Port written to.\n");
 TestData = ParPortIn();
 TestData = TestData ^ SenseBitsInvert;
 printf("Parallel Port Data = %2x  \n", TestData);

 return(0);
} //END MAIN --------------------------------------------------


/*------------------( FUNCTION CODE )-------------------------------------*/

/*-------[ Function: ParPortInit - Sets ParPortAddr ]--------------------*/
int ParPortInit()
{
 ParPortAddr     = LPT1; //!! Fix this to take command line & environment var !!
 PrtStatAddr     = ParPortAddr + PRINTERSTATUS;
 PrtCtrlAddr     = ParPortAddr + PRINTERCONTROL;
 BitsNowOn       = 0;
 SenseBitsInvert = SENSEBITSDEFAULT;

 printf("ParPortInit Done\n");

 return(0);
}

/*-------[ Function: ParPortOut - Writes to ParPort base addr ]----------*/
void ParPortOut(unsigned char OutData)
{
 outp(ParPortAddr, OutData);
 BitsNowOn = OutData;
}



/*-------[ Function BitOff  - Turns Off one bit ]-------------------------*/
int BitOff(unsigned char BitNum)
{
unsigned char  OutData;

 if (BitNum > MAXBITNUM ) return(ERR_BADBITNUM);

 OutData = 0x01;
 OutData = OutData << BitNum;
 OutData = ~OutData;
 OutData = OutData & BitsNowOn;
 BitsNowOn = OutData;
 ParPortOut(OutData);

 return(0);
}

/*-------[ Function: BitOn - Turns On one bit ]-------------------------*/
int BitOn(unsigned char BitNum)
{
unsigned char  OutData;

 if (BitNum > MAXBITNUM ) return(ERR_BADBITNUM);

 OutData = 0x01;
 OutData = OutData << BitNum;
 OutData = OutData | BitsNowOn;
 BitsNowOn = OutData;
 ParPortOut(OutData);

 return(0);
}


/*---------------( ParPortIn   Function )------------------------------------*/
/* Reads both 'Printer Status' and 'Printer Control' (readback) bytes and    */
/* assembles a single 8-bit input byte with inversions fixed etc.            */
/*---------------------------------------------------------------------------*/

unsigned char ParPortIn()
{

unsigned char StatPort;
unsigned char CtrlPort;
unsigned char PortWork;

 outp(PrtCtrlAddr, 0x04);        /* Set output port to prepare for input     */
                                 /* (Bit Position)     7 6 5 4   3 2 1 0     */
 StatPort = inp(PrtStatAddr)   ; /* (Bit Locations)    1 0 2 3   5           */
 CtrlPort = inp(PrtCtrlAddr)   ; /* (Bit Locations)              7 6 4       */
 PortWork = 0;
 PortWork = StatPort >> 6 ;                             /*Get Bit 0 and 1*/
 PortWork = PortWork | ((StatPort & 0x20) >> 3) ;       /*Get Bit 2*/
 PortWork = PortWork | ((StatPort & 0x10) >> 1) ;       /*Get Bit 3*/
 PortWork = PortWork | ((CtrlPort & 0x02) << 3) ;       /*Get Bit 4*/
 PortWork = PortWork | ((StatPort & 0x08) << 2) ;       /*Get Bit 5*/
 PortWork = PortWork | ((CtrlPort & 0x0C) << 4) ;       /*Get Bit 6 & 7*/
 PortWork = PortWork ^ DIINVERT;                        /*Fix Inverted Bits*/
 return(PortWork);

} // END ParPortIn


