{---------------------------------------------------------------------------}
{ PPORTBIT.PAS                                                              }
{ PROGRAMMER:   Terry King                                                  }
{ WHAT IT DOES: Turns Bits on and off on Parallel Port from command line    }
{ 04/02/99 - 1.00  Initial Try                                              }
{                                                                           }
{---------------------------------------------------------------------------}

USES Crt, Hexutil;

      {------( PUBLIC CONSTANTS )------------------------}
CONST
MAXBITPOS     =  7;              {Max Bit position in a 8-bit byte }

{-------( IBM type Parallel Printer Port addresses )------------------------}
LPT1               = $0378;    {PARALLEL card base address }
LPT2               = $0278;    {PARALLEL card base address }
LPTM               = $03BC;    {PARALLEL card base address }
DataLatch          = $0000;    {ADDER To get to DO "DATA LATCH" register }
PrinterStatus      = $0001;    {ADDER To get to DI "PRINTER STATUS" register }
PrinterControl     = $0002;    {ADDER To get to DO "PRINTER CONTROL" register }



      {------( PUBLIC TYPES     )------------------------}
TYPE

OUTBITPOS     =  0..7;              {Type for a Bit position in a 8-bit byte }
INBITPOS      =  0..7;              {Type for a Bit position in PAR PORT input}

{---------------------------------------------------------------------------}
{               V A R I A B L E S: FOR YOUR MACHINE DESIGN                  }
{---------------------------------------------------------------------------}
VAR {MainLine}
    Debug      : Boolean;
    BitsNowON  : Byte;
    Bit2Hit    : Byte;
    Bitval     : Word;
    Mask       : Word;
    Parm1Str   : string;     { Parameter }
    Parm2Str   : string;     { Parameter }
    Parm3Str   : string;     { Parameter }
    Parm4Str   : string;     { Parameter }
    Parm5Str   : string;     { Parameter }

    tbuff      : string;     { Strings are groups of characters        }
    n          : integer;    { Integers are whole numbers up to 32,767 }
    Error      : Boolean;
    ActionError: Boolean;
    Bit        : Boolean;
    RtnCode    : Word;

    PortAddrString : String;   {String showing port address being used}
  { Note these addresses are  calculated at INIT time }
  CardBase        : Word;
  DOAddr          : word;   {Main Digital Output Address }
  PrtStatAddr     : word;   {Input from "Printer Status' }
  PrtCtrlAddr     : word;   {'Control Port' Input/Output Address }



{---------------------------------------------------------------------------}
{   PROCEDURES AND FUNCTIONS                                                }
{---------------------------------------------------------------------------}

{---------------( WriteDO      Procedure )----------------------------------}
{ Selects PAR  Digital Out register, Writes it.                             }
{---------------------------------------------------------------------------}

Procedure WriteDO(WriteData: byte);

BEGIN
  Port[DOAddr] := WriteData; {Write  DIDO Device Data }
END;


{---------------(BitOn    Procedure)----------------------------------------}
{ Sets a Digital Out Bit ON by it's BitPos                                  }
{---------------------------------------------------------------------------}
PROCEDURE BitOn(BitToSet: OUTBITPOS);

VAR  BitMask: byte;

BEGIN
  BitsNowON    := Port[DOAddr];
  BitMask      := $0001 shl BitToSet;
  BitsNowOn    := BitsNowOn or BitMask;     {BitsNowOn is Global}
  Port[DOAddr] := BitsNowOn; {Write  DIDO Device Data }
END;

{---------------(BitOff   Procedure)----------------------------------------}
{ Resets a Digital Out Bit by it's BitPos                                   }
{---------------------------------------------------------------------------}
PROCEDURE BitOff(BitToReset: OUTBITPOS);

VAR  BitMask: byte;

BEGIN
  BitsNowON    := Port[DOAddr];
  BitMask   := (($0001 shl BitToReset) xor $FF);
  BitsNowOn := BitsNowOn and BitMask;     {BitsNowOn is Global}
  Port[DOAddr] := BitsNowOn; {Write  DIDO Device Data }
END;

{---------------(BitFlip  Procedure)----------------------------------------}
{ Inverts a Digital Out Bit by it's BitPos                                  }
{---------------------------------------------------------------------------}
PROCEDURE BitFlip(BitToSet: OUTBITPOS);

VAR  BitMask: byte;

BEGIN
  BitsNowON    := Port[DOAddr];
  BitMask      := $0001 shl BitToSet;
  IF ((BitsNowON AND BitMask) <> 0)
    THEN {Bit Is On}
      BitOff(BitToSet)
    ELSE
      BitOn(BitToSet);
END;

{---------------(DIORESET Procedure)----------------------------------------}
{ Resets DACA Digital Out register to predetermined value ($FFFF for OPTO22)}
{---------------------------------------------------------------------------}
PROCEDURE DioReset;

BEGIN
  WriteDO($00);       {Write  DIDO Device to Initialize}
END;

{-----------( Pulse    Procedure )-------------------------------------------}
{ Blinks a light on and off                                                  }
{----------------------------------------------------------------------------}

PROCEDURE Pulse(Light: OUTBITPOS; NumTimes: Integer; TimeOn: Integer );

VAR  Interval, n : Integer;

BEGIN
  Interval   := TimeOn;
  FOR n := 1 to Numtimes DO
    BEGIN
      BitOn(Light);
      delay(Interval);
      BitOff(Light);
      delay(Interval * 2);
    END;
END;

{---------------------------------------------------------------------------}
{ MAIN PROGRAM:                                                             }
{---------------------------------------------------------------------------}
BEGIN
  Debug := FALSE;

{-----( Get Parameters                                      )-------------}
{-----( Assumed: PARPORT gets the first parameter           )-------------}


 IF ParamCount >= 3 THEN
   BEGIN
     Parm1Str := ParamStr(1);   {Port}
     Parm2Str := ParamStr(2);   {Bit}
     Parm3Str := ParamStr(3);   {Action}
     Parm4Str := ParamStr(4);   {Option}
     Parm5Str := ParamStr(5);   {??}
     IF (Parm4Str = 'DEBUG') OR (Parm4Str = 'debug') THEN Debug := TRUE;
     IF Debug THEN
       BEGIN
         Writeln('COMMAND LINE PARAMETERS:');
         Writeln('PORT    = ', Parm1Str);
         Writeln('PORTADDR= ', PortAddrString);
         Writeln('BIT     = ', Parm2Str);
         Writeln('ACTION  = ', Parm3Str);
         Writeln('OPTION  = ', Parm4Str);
       END;

{-----( INITIALIZE ADDRESSES )----------------------------------------------}
  Cardbase := LPT1;   {Initial default}

  IF (Upper(Parm1Str) = 'LPT1') THEN CardBase := LPT1;
  IF (Upper(Parm1Str) = 'LPT2') THEN CardBase := LPT2;
  IF (Upper(Parm1Str) = 'LPTM') THEN CardBase := LPTM;

  DOAddr         := Cardbase + DataLatch;
  PrtStatAddr    := Cardbase + PrinterStatus;
  PrtCtrlAddr    := Cardbase + PrinterControl;
  PortAddrString := Word2Hex(Cardbase);

{-----( HANDLE PARAMETERS FOR BIT, ACTION )---------------------------------}
   Error         := FALSE;
   ActionError   := FALSE;

   Val(Parm2Str, Bit2Hit, RtnCode);
   IF (RtnCode <> 0) THEN
     BEGIN
       IF (Upper(Parm2Str) = 'CLEAR')
       THEN
         BEGIN
           WriteDO($00);
           Error  := TRUE;
         END
       ELSE
       WriteLn('***ERROR in BIT NUMBER PARAMETER***');
       Error  := TRUE;
     END;

   IF NOT Error THEN
     BEGIN
       IF (Bit2Hit < 0) OR (Bit2Hit > MAXBITPOS) THEN
         BEGIN
           WriteLn('*** ERROR: BIT NUMBER MUST BE FROM 0 to 7 ***');
           Error  := TRUE;
         END;
     END;

   IF NOT Error THEN
     BEGIN
       ActionError := TRUE;  {Reset if OK}
       IF (Upper(Parm3Str) = 'ON') THEN
         BEGIN
           BitOn(Bit2Hit);
           ActionError := FALSE;
         END;
     END;

   IF NOT Error THEN
     BEGIN
       IF (Upper(Parm3Str) = 'OFF') THEN
         BEGIN
           BitOff(Bit2Hit);
           ActionError := FALSE;
         END;
     END;

   IF NOT Error THEN
     BEGIN
       IF (Upper(Parm3Str) = 'FLIP') THEN
         BEGIN
           BitFlip(Bit2Hit);
           ActionError := FALSE;
         END;
     END;

   IF ActionError THEN
     BEGIN
       Error := TRUE;
       WriteLn('*** ERROR: ACTION MUST BE "ON" "OFF" OR "FLIP" ***')
     END;

   END{OK: Do It }
 ELSE  {Wrong number of parameters alltogether ! }
   BEGIN
     Writeln('***********************************************************************');
     Writeln('PPORTBIT (c) 1999 by Mountain ClockWorks Box 232K West Topsham, VT 05086');
     Writeln('All rights reserved. NonCommercial & Educational use permitted.');
     Writeln('*** ERROR or NO PARAMETERS SPECIFIED ***');
     Writeln('USEAGE:  PPORTBIT <Port> <Bit> <ON|OFF|FLIP>');
     Writeln('NOTE <Port> Must be LPT1 (x0378) or LPT2 (x0278) or LPTM (x03BC) ');
     Writeln('NOTE <Bit > Must from Zero (0) to Seven (7) ');
     Writeln('NOTE <Action> Must be ON or OFF or FLIP (Invert to opposite value) ');
     Writeln('EXAMPLES: PPORTBIT LPT1 3 ON    PPORTBIT LPT2 OFF  PPORTBIT LPTM FLIP');
     Writeln('ALSO: You can do: "PPORTBIT LPTx CLEAR ALL  to reset all bits');
     Writeln('EMAIL: tking@together.net UPDATES: http://homepages.together.net/~tking');
   END;


  {---------( END OF  PROGRAM )------------------------------------}


END.

