{---------------------------------------------------------------------------}
{ Strobe Program:  STROBE.PAS                                               }
{ DATE:         (See Version, below)                                        }
{ PROGRAMMER:   Terry King                                                  }
{ WHAT IT DOES: Flashes LED's or Xenon Strobe Lights at accurate            }
{               Intervals                                                   }
{ HISTORY:                                                                  }
{ 01/25/97 - Update: set level to 1.00: Latest                              }
{ 01/28/97 - BIG Numerals on screen                                         }
{ 01/28/97 - Ctrl-Break out of loops                                        }
{ 02/17/97 - Clean Up                                                       }
{                                                                           }
{---------------------------------------------------------------------------}

{---------------------------------------------------------------------------}
{               U S E S :  External UNITS with needed functions             }
{---------------------------------------------------------------------------}

USES  dos, crt,   {Standard Borland TURBO units}
      CtrlBrk,    {Ctrl-Break handling}
      ParPort,    {Parallel port I/O Support}
      KbdInput,   {ReadLnPF support for text and keystrokes}
      HRTimers,   {High Resolution Timers}
       PCTimers    {Timer trigger support}
    {  Hexutil};    {Formatting utilities, Sounds}

{---------------------------------------------------------------------------}
{               T Y P E S :                                                 }
{----------------------------------------------------------------------------}

TYPE

SegLayout = ARRAY[0..11, 1..7] OF Byte;  {For 7-segment display}


{---------------------------------------------------------------------------}
{               C O N S T A N T S:                                          }
{---------------------------------------------------------------------------}
CONST    {NOTE: KeyStroke constants are in KBDINPUT.PAS}

Version       = '1.100 [02/17/97: Big Numbers] Terry King '; {++Program version}

CHARCOLOR       = Yellow;   {Our screen colors}
BACKCOLOR       = Blue;
SEGON           = LightGreen;
SEGOFF          = Blue;
BLINKIT         = 128;
NOBLINK         = 0;

WAIT            = FALSE;  {For ReadLnPF}
NOWAIT          = TRUE;

            {-------( 7 segment Display Layout )----------}
            {     A     }
            {   F   B   }
            {     G     }
            {   E   C   }
            {     D     }
                                 {A B C D E F G}
SegArray        : SegLayout = (  (1,1,1,1,1,1,0),    {Numeral 0}
                                 (0,1,1,0,0,0,0),    {1}
                                 (1,1,0,1,1,0,1),    {2}
                                 (1,1,1,1,0,0,1),    {3}
                                 (0,1,1,0,0,1,1),    {4}
                                 (1,0,1,1,0,1,1),    {5}
                                 (1,0,1,1,1,1,1),    {6}
                                 (1,1,1,0,0,0,0),    {7}
                                 (1,1,1,1,1,1,1),    {8}
                                 (1,1,1,0,0,1,1),    {9}
                                 (5,13,13,5,0,0,5),  {X location}
                                 (0,0,6,10,6,0,5)  );{Y location}


{-------( Digital IN Bits: What they are being used for:  )--- -------------}
In0            =  $01;      { Pin 10 on Parallel Port }
In1            =  $02;      { Pin 11 on Parallel Port }
BotSensor      =  $04;      { Pin 12 on Parallel Port }
TopSensor      =  $08;      { Pin 13 on Parallel Port }
In4            =  $10;      { Pin 14 on Parallel Port }
In5            =  $20;      { Pin 15 on Parallel Port }
In6            =  $40;      { Pin 16 on Parallel Port }
In7            =  $80;      { Pin 17 on Parallel Port }

SensorPolarity =  $00;      {Opto sensors are normally '1', go to '0' in light}

{-------( Digital OUT Bits: What they are being used for:  )------  }
Out0        = 0;                                  { Pin 2  on Parallel Port }
Out1        = 1;                                  { Pin 3  on Parallel Port }
Out2        = 2;                                  { Pin 4  on Parallel Port }
BotBeamLED  = 3;                                  { Pin 5  on Parallel Port }
StrobeLED1  = 0;                                  { Pin 6  on Parallel Port }
StrobeLED2  = 1;                                  { Pin 7  on Parallel Port }
BlinkLED    = 6;                                  { Pin 8  on Parallel Port }
TopBeamLED  = 7;                                  { Pin 9  on Parallel Port }
{---------------------------------------------------------------------------}
{               V A R I A B L E S:                                          }
{---------------------------------------------------------------------------}
VAR {MainLine}


PgmDescription : String[50];              {Saved with file, shown at load time}

ResponseType   : ReadlnPF_TYPE;           {For use by KbdInput Unit }
KeyCode        : KeyCodeTYPE;
UserString     : CommandStringTYPE;
Statement      : CommandStringTYPE;

{-----------------( SCALAR VARIABLES )----------------------------}
KbChar           : char;                    { for kbd input }

InputByte        : Byte;                    {Data read In/Out each SampleTime}
OutputByte       : Byte;
OutputByteInit   : Byte;                    {Startup time value}

Xnow, Ynow       : Byte;                    {Cursor location}

n                : word;                    {general purpose}
SampleTime       : word;                    {Sample period (Ms)}

StrobeDuration   : word;                    {Ms}
StrobePeriod     : word;
StrobeFrequency  : real;
WorkFrequency    : real;

StrobeFreqTenths : real;                    {For character display}
StrobeFreq1s     : real;
StrobeFreq10s    : real;
StrobeFreq100s   : real;

StrobeCycles     : word;

Samples          : Longint;                 {Samples of I/O }

Done             : Boolean;                 {End main loop when Done}
TriggerStrobe    : Boolean;
UpDateScreen     : Boolean;
PhotoTrig        : Boolean;
Bit              : Boolean;

TopTrigger  : TRIGGER_REC_TYPE;  {Top Sensor}
BotTrigger  : TRIGGER_REC_TYPE;  {bottom Sensor}
{Time1,Time2 : HRTime_Rec_type;}


(*Ŀ
   INTERNAL PROCEDURES and FUNCTIONS                                       
  *)

PROCEDURE StrobeAction(Period: word; Duration: word; Cycles: word);

VAR

  OffTime : word;

BEGIN

  OffTime := Period - Duration;
  IF (OffTime < 0) THEN Offtime := 0;

  FOR n := 1 to Cycles DO
    BEGIN
      WriteDO($FF);             {Strobe LED's ON}
      _Delay(Duration);
      WriteDO($00);             {Strobe LED's OFF}
      _Delay(OffTime);
    END;

END;{Strobeaction}


{-----------------------------------------------------------------------}
PROCEDURE VertBlob(X: byte; Y: Byte; Bit: Boolean);

VAR

XNow, Ynow: Byte;
Column    : word;
Row       : word;

BEGIN

  FOR Row := 0 to 5 DO
    BEGIN
      GoToXY(X,Y + Row);
      FOR Column := 1 to 5 DO
        BEGIN
          IF Bit
          THEN
            BEGIN
              TextColor(SEGON);
              Write('')
            END
          ELSE
            BEGIN
             TextColor(SEGOFF);
             Write('');
            END;
        TextColor(Yellow);
        END;
    END;

END;


{-----------------------------------------------------------------------}
PROCEDURE HorzBlob(X: byte; Y: Byte; Bit: Boolean);

VAR

XNow, Ynow: Byte;
Column    : word;
Row       : word;

BEGIN

  FOR Row := 0 to 1 DO
    BEGIN
      GoToXY(X,Y + Row);
      FOR Column := 1 to 8 DO
        BEGIN
          IF Bit
          THEN
            BEGIN
              TextColor(SEGON);
              Write('')
            END
          ELSE
            BEGIN
             TextColor(SEGOFF);
             Write('');
            END;
        TextColor(Yellow);
        END;
    END;

END;


{-----------------------------------------------------------------------}
PROCEDURE Paint7Seg(Value: integer; X: byte; Y: Byte);

VAR

Segment    : Byte;
Xnow, Ynow : Byte;
SegOn      : Boolean;

BEGIN

  IF (Value < 0) OR (Value > 9) THEN halt(3);
  Xnow := WhereX;
  Ynow := WhereY;
  FOR Segment := 1 to 7 DO
    BEGIN
      IF (SegArray[Value, Segment] = 1)
        THEN SegOn := TRUE ELSE SegOn := FALSE;
        IF    (Segment = 1)
          OR  (Segment = 4)
          OR  (Segment = 7)
          THEN HorzBlob( (Xnow + Segarray[10, Segment]),
                         (Ynow + Segarray[11, Segment]),
                         SegOn );
        IF    (Segment = 2)
          OR  (Segment = 3)
          OR  (Segment = 5)
          OR  (Segment = 6)
          THEN VertBlob( (Xnow + Segarray[10, Segment]),
                         (Ynow + Segarray[11, Segment]),
                         SegOn );
    END;

END;{Paint7Seg}

(*Ŀ
   MAIN PROGRAM:                                                           
  *)
BEGIN

  {---------( INITIALIZE )-----------------------------------------}
  BreakFlag := FALSE;  {Set up to allow Ctrl-Break to exit tight loops }
  SetCtrlBrk;

  Done             := FALSE;
  TriggerStrobe    := FALSE;

  Samples          := 0;
  SampleTime       := 50;                 {Milliseconds}

  StrobeCycles     := 1;
  StrobePeriod     := 250;
  StrobeDuration   := 1;

  TriggerSetup(TopTrigger, TopSensor, SensorPolarity, 1);
  TriggerSetup(BotTrigger, BotSensor, SensorPolarity, 1);

  WriteDO($00);             {Set parallel port bits all OFF }
  {---------( INITIALIZE THE SCREEN )------------------------------}

  TextBackground(BACKCOLOR);
  TextColor(CHARCOLOR);
  ClrScr;
  Write('     S T R O B E  PROGRAM  ', Version);
  GoToXY(1,24);
  Write('ESC to EXIT');

   UpDateScreen := TRUE;

  REPEAT {-----------------( MAIN LOOP IN PROGRAM )-----------------------}

   TriggerStrobe := FALSE;

   Delay(SampleTime);
   INC(Samples);
   GoToXY(1,25);
   Write('Loops = ', Samples:6 , '     Port Address = X', PortAddrString);
  {---------( GET USER INPUT, CHECK, HANDLE DIFFERENT TYPES )------}
   UserString := '';
   KeyCode    := 0;
   ResponseType := ReadlnPF(UserString, KeyCode, TRUE );  {Get user input}
   IF (ResponseType <> NoKey) THEN
     BEGIN
       IF (ResponseType = CommandStr) THEN
         BEGIN
           Statement := UserString;
           UserString := '';
         END;
     {------------( Look for ESC Key )-------------------------}

       IF (ResponseType = Esc) THEN Done := TRUE;

     {------------( Look for Extended Keys )-------------------------}

       IF (ResponseType = ExtendedKey) THEN
         BEGIN
           GoToXY(60,1);
           IF (KeyCode = UPARROW) THEN
             BEGIN
               UpdateScreen := TRUE;
               INC(StrobePeriod);
             END;
           IF (KeyCode = DOWNARROW) THEN
             BEGIN
               UpdateScreen := TRUE;
               DEC(StrobePeriod);
             END;

           IF (KeyCode = PGUP     ) THEN
             BEGIN
               UpdateScreen := TRUE;
             END;

           IF (KeyCode = PGDN     ) THEN
             BEGIN
               UpdateScreen := TRUE;
             END;


           IF (KeyCode = INS      ) THEN ;
           IF (KeyCode = DEL      ) THEN ;
           IF (KeyCode = HOME     ) THEN
             BEGIN
               UpdateScreen := TRUE;
               INC(StrobePeriod, 10);
             END;
           IF (KeyCode = _END     ) THEN
             BEGIN
               UpdateScreen := TRUE;
               DEC(StrobePeriod, 10);
             END;
         END;{Extended Keys}

     {------------( Look for PF Keys )-------------------------}

       IF (ResponseType = PFKEY) THEN
         BEGIN
           UpdateScreen := TRUE;
           GoToXY(50,4);
           IF (KeyCode = F1       ) THEN TriggerStrobe := TRUE;

           IF    (KeyCode = F2       )
              OR (KeyCode = F3       )            THEN
             REPEAT
               BEGIN
                 GoToXY(50,4);
                 write('F2: WAITING FOR TRIGGER');
                 BitON(TopBeamLED);
                 BitON(BotBeamLED);
                 BreakFlag := FALSE;
                 REPEAT UNTIL (Triggered(TopTrigger) OR BreakFlag);

                 StrobeAction(StrobePeriod, StrobeDuration, StrobeCycles);

                 GoToXY(50,4);
                 write('                         ');
               END;
             UNTIL (KeyCode = F2) OR KeyPressed;

           IF (KeyCode = F4       ) THEN
             REPEAT
               BEGIN
                 StrobeAction(StrobePeriod, StrobeDuration, StrobeCycles);
               END;
             UNTIL KeyPressed;
           IF (KeyCode = F5       ) THEN DEC(StrobePeriod);
           IF (KeyCode = F6       ) THEN INC(StrobePeriod);
           IF (KeyCode = F7       ) THEN DEC(StrobePeriod, 10);
           IF (KeyCode = F8       ) THEN INC(StrobePeriod, 10);
           IF (KeyCode = F9       ) THEN DEC(StrobeCycles);
           IF (KeyCode = F10      ) THEN INC(StrobeCycles);
         END;{PFKeys}

     END;{Not NoKey}


{----( PUT OUT BIG TIMING CHARACTERS )---}
  IF UpdateScreen THEN
    BEGIN
      {---( Check, fix illegal values )----}
      IF (StrobePeriod < 50) THEN StrobePeriod := 50;
      IF (StrobeCycles = 0)  THEN StrobeCycles := 1;
      {---( Calculate values for large character display )----}
      StrobeFrequency :=    ( 1.0 / StrobePeriod) * 1000.0;
      WorkFrequency    := StrobeFrequency;
      StrobeFreq100s   := Trunc(WorkFrequency / 100.0);
      WorkFrequency    := WorkFrequency - (StrobeFreq100s * 100);
      StrobeFreq10s    := Trunc(WorkFrequency / 10.0);
      WorkFrequency    := WorkFrequency - (StrobeFreq10s * 10);
      StrobeFreq1s     := Trunc(WorkFrequency  / 0.99999999);
      WorkFrequency    := WorkFrequency - (StrobeFreq1s);
      StrobeFreqTenths := Trunc(WorkFrequency  / 0.10);

      GoToXY(3,11);
      Paint7Seg(Round(StrobeFreq10s),  WhereX, WhereY );

      GoToXY(25,11);
      Paint7Seg(Round(StrobeFreq1s),  WhereX, WhereY );

      GoToXY(51,11);
      Paint7Seg(Round(StrobeFreqTenths),  WhereX, WhereY );

      TextColor(SEGON);

      {---( DECIMAL POINT! )---}
      GoToXY(45,22); Write('');  GoToXY(46,21); Write('');
      GoToXY(45,21); Write('');  GoToXY(47,21); Write('');
      GoToXY(46,22); Write('');  GoToXY(48,22); Write('');
      GoToXY(47,22); Write('');  GoToXY(48,21); Write('');

      {---( LABEL UNITS )---}
      GoToXY(71,12); Write('FLASHES');
      GoToXY(71,13); Write('  PER  ');
      GoToXY(71,14); Write('SECOND ');
      GoToXY(71,22); Write('(HERTZ)');
      TextColor(CHARCOLOR);
    END;

   IF UpdateScreen THEN
     BEGIN
       GoToXY(1,2);
       Write('F1: MANUAL STROBE TRIGGER');
       GoToXY(1,3);
       Write('F2: PHOTOGATE TRIGGERED STROBE');
       GoToXY(1,4);
       Write('F3: REPEATED PHOTOGATE TRIGGERED STROBE: Q QUITS');
       GoToXY(1,5);
       Write('F4: CONTINUOUS STROBE (Q QUITS) ');

       GoToXY(1,7);
       Write('StrobePeriod   = ', StrobePeriod:4,   '  Ms  [ -  Downarrow  + UpArrow ]' );
       GoToXY(1,8);
       Write('                     ',               '  Ms  [ -- End        ++ Home ]');
       GoToXY(1,9);
       Write('StrobeCycles   = ', StrobeCycles:4,   '      [LESS:F9  MORE:F10]');
     END;{UpdateScreen}


 {-------( EXECUTE EACH TIME THRU LOOP )--------------------- }
   IF TriggerStrobe THEN
    BEGIN
      StrobeAction(StrobePeriod, StrobeDuration, StrobeCycles);
      TriggerStrobe := FALSE;
    END;
    UpdateScreen := FALSE;
 {--( END OF MAIN PROGRAM LOOP )---}
  UNTIL Done;


  {---------( END OF  PROGRAM )------------------------------------}

  ResetCtrlBrk;                          {MUST Do this or Bad Stuff!!}
  Textmode(Co80);                        {Put the Display back to normal }
END.
