{---------------------------------------------------------------------------}
{ BitMachine Program:  TALK2ME.PAS                                            }
{ PROGRAMMER:   Terry King                                                  }
{ WHAT IT DOES: Constructs, runs Dialogs between computer and users         }
{ HISTORY:                                                                  }
{ 03/25/97 - First Attempt                                                  }
{ 03/25/97 -                                                                }
{                                                                           }
{                                                                           }
{---------------------------------------------------------------------------}

{---------------------------------------------------------------------------}
{               U S E S :  External UNITS with needed functions             }
{---------------------------------------------------------------------------}

USES  dos, crt,   {Standard Borland TURBO units}
      TP_VCO,     {IBM Voice Communications Option support}
      {CtrlBrk,}    {Ctrl-Break handling}
      ParPort,    {Parallel port I/O Support}
      KbdInput,   {ReadLnPF support for text and keystrokes}
      MWindow,    { DOS Window support }
      Sounds;     {Sound effects}
    {  Hexutil}    {Formatting utilities, Sounds}
{---------------------------------------------------------------------------}
{               C O N S T A N T S:                                          }
{---------------------------------------------------------------------------}
CONST    {NOTE: KeyStroke constants are in KBDINPUT.PAS}

Version       = '1.000 [02/25/97: 1st Try    ] Terry King '; {++Program version}

CHARCOLOR       = Yellow;   {Our screen colors}
BACKCOLOR       = Blue;
BLINKIT         = 128;
NOBLINK         = 0;

INFO          = 1     ; {WindowID's}
USER          = 2     ;
STATUS        = 3     ;


MSGFIELDSIZE    = 43;

WAIT            = FALSE;  {For ReadLnPF}
NOWAIT          = TRUE;


{-------( Digital IN Bits: What they are being used for:  )--- -------------}
CueSwitch      =  $01;      { Pin 10 on Parallel Port }
In1            =  $02;      { Pin 11 on Parallel Port }
In2            =  $04;      { Pin 12 on Parallel Port }
BackUpSwitch   =  $08;      { Pin 13 on Parallel Port }
In4            =  $10;      { Pin 14 on Parallel Port }
In5            =  $20;      { Pin 15 on Parallel Port }
In6            =  $40;      { Pin 16 on Parallel Port }
In7            =  $80;      { Pin 17 on Parallel Port }

SensorPolarity =  $00;      {Opto sensors are normally '1', go to '0' in light}

{-------( Digital OUT Bits: What they are being used for:  )------  }
Out0        = 0;                                  { Pin 2  on Parallel Port }
Out1        = 1;                                  { Pin 3  on Parallel Port }
Out2        = 2;                                  { Pin 4  on Parallel Port }
Out3        = 3;                                  { Pin 5  on Parallel Port }
Out4        = 4;                                  { Pin 6  on Parallel Port }
CueLED      = 5;                                  { Pin 7  on Parallel Port }
Out6        = 6;                                  { Pin 8  on Parallel Port }
Out7        = 7;                                  { Pin 9  on Parallel Port }

{---------------------------------------------------------------------------}
{               T Y P E S :                                                 }
{---------------------------------------------------------------------------}

TYPE

FileNameTYPE    = String[50];  {File Name Strings}
MaxString       = String[100];
MsgString       = String[MSGFIELDSIZE];

{---------------------------------------------------------------------------}
{               V A R I A B L E S:                                          }
{---------------------------------------------------------------------------}
VAR {MainLine}


PgmDescription : String[50];              {Saved with file, shown at load time}

ResponseType   : ReadlnPF_TYPE;           {For use by KbdInput Unit }
KeyCode        : KeyCodeTYPE;
UserString     : CommandStringTYPE;
Statement      : CommandStringTYPE;

{----------( DISK FILE VARIABLES )--------------------------------}

PgmFile        : Text;                    {Program load/save}
PgmFileName    : FileNameTYPE;
OldPgmFileName : FileNameTYPE;

{----------( STRING VARIABLES )--------------------------------}

TalkString     : MaxString;
UtilStr        : MaxString;
InWorkStr      : MaxString;               {Used in handling Input Data}
InDataStr      : MaxString;
OutWorkStr     : MaxString;               {Used in handling Output Data}
OldOutStr      : MaxString;
OutDataStr     : MaxString;

Msg            : MsgString;

{-----------------( SCALAR VARIABLES )----------------------------}
KbChar           : char;                    { for kbd input }

InputByte        : Byte;                    {Data read In/Out each SampleTime}
OutputByte       : Byte;
OutputByteInit   : Byte;                    {Startup time value}

Xnow, Ynow       : Byte;                    {Cursor location}

n                : word;                    {general purpose}
SampleTime       : word;                    {Sample period (Ms)}

Samples          : Longint;                 {Samples of I/O }

Done             : Boolean;                 {End main loop when Done}
UpDateScreen     : Boolean;
Bit              : Boolean;


(*Ŀ
   INTERNAL PROCEDURES and FUNCTIONS                                       
  *)

(*Ŀ
   MAIN PROGRAM:                                                           
  *)
BEGIN

  {---------( INITIALIZE )-----------------------------------------}
(*BreakFlag := FALSE;  {Set up to allow Ctrl-Break to exit tight loops }*)
(*SetCtrlBrk; *)

  Done             := FALSE;

  Samples          := 0;
  SampleTime       := 50;                 {Milliseconds}
  UpDateScreen     := TRUE;

  WriteDO($00);             {Set parallel port bits all OFF }
  {---------( INITIALIZE THE SCREEN )------------------------------}

  ClrScr;
  WindowFrame(1,  1,  80, 10,
    (' T A L K  2  M E !' + Version +  ' ' ),
                                            Yellow , Black, INFO);
  WindowFrame(1, 10,  80, 21,
    (' TEXT ' ),
                                            White  , Green, USER);
  WindowFrame(1,  21, 80, 25, ' STATUS ' , White  , Blue , STATUS);
(*
  TextBackground(BACKCOLOR);
  TextColor(CHARCOLOR);
*)
  SelectWindow(STATUS);

   TalkString  := 'Oh, Hello. What''s happening?';
   UserString := '';

  REPEAT {-----------------( MAIN LOOP IN PROGRAM )-----------------------}

   Delay(SampleTime);
   INC(Samples);

   SelectWindow(STATUS);
   GoToXY(1,2);
   Write('Loops = ', Samples:6 , '     Port Address = X', PortAddrString);

  {---------( GET USER INPUT, CHECK, HANDLE DIFFERENT TYPES )------}
(*   UserString := '';*)
   SelectWindow(USER);
   KeyCode    := 0;
   ResponseType := ReadlnPF(UserString, KeyCode, TRUE );  {Get user input}
   IF (ResponseType <> NoKey) THEN
     BEGIN
       IF (ResponseType = CommandStr) THEN
         BEGIN
           TalkString := UserString + '.';
           Say(TalkString);
           UserString := '';
           Writeln;
           UpDateScreen := TRUE;
         END;
     {------------( Look for ESC Key )-------------------------}

       IF (ResponseType = Esc) THEN Done := TRUE;

     {------------( Look for Extended Keys )-------------------------}

       IF (ResponseType = ExtendedKey) THEN
         BEGIN
           GoToXY(60,1);
           IF (KeyCode = UPARROW) THEN
             BEGIN
               Say(TalkString);
               UpdateScreen := TRUE;
             END;
           IF (KeyCode = DOWNARROW) THEN
             BEGIN
               UpdateScreen := TRUE;
             END;

           IF (KeyCode = PGUP     ) THEN
             BEGIN
               UpdateScreen := TRUE;
             END;

           IF (KeyCode = PGDN     ) THEN
             BEGIN
               UpdateScreen := TRUE;
             END;


           IF (KeyCode = INS      ) THEN ;
           IF (KeyCode = DEL      ) THEN ;
           IF (KeyCode = HOME     ) THEN
             BEGIN
               UpdateScreen := TRUE;
             END;
           IF (KeyCode = _END     ) THEN
             BEGIN
               UpdateScreen := TRUE;
             END;
         END;{Extended Keys}

     {------------( Look for PF Keys )-------------------------}

       IF (ResponseType = PFKEY) THEN
         BEGIN
           UpdateScreen := TRUE;
           GoToXY(50,4);
           IF (KeyCode = F1   ) THEN VoiceSpeed(Slowest);
           IF (KeyCode = F2   ) THEN VoiceSpeed(Slower);
           IF (KeyCode = F3   ) THEN VoiceSpeed(Normal);
           IF (KeyCode = F4   ) THEN VoiceSpeed(Faster);

           IF (KeyCode = F5   ) THEN VoicePitch(Whisper);
           IF (KeyCode = F6   ) THEN VoicePitch(Lower);
           IF (KeyCode = F7   ) THEN VoicePitch(Normal);
           IF (KeyCode = F8   ) THEN VoicePitch(Higher);

           IF (KeyCode = F9   ) THEN VoiceLoudness(Soft);
           IF (KeyCode = F10  ) THEN VoiceLoudness(Loudest);
         END;{PFKeys}

     END;{Not NoKey}


{----( SHOW SCREEN )---------------------}

   IF UpdateScreen THEN
     BEGIN
       SelectWindow(INFO);
       GoToXY(1,1);
       Write('F1 -- F4 SLOWER -- FASTER');
       GoToXY(1,2);
       Write('F5 -- F8 LOWER  -- HIGHER PITCH');
       GoToXY(1,3);
       Write('F9 -- SOFT  F10 -- LOUD');
       GoToXY(1,4);
       Write('(UP ARROW TO REPLAY)');
       GoToXY(1,5);
       Write('    Hit <ESC> to Quit   ');
       GoToXY(1,6);
       Write('Just type sentences in to try stuff out');
       GoToXY(1,7);

     END;{UpdateScreen}


 {-------( EXECUTE EACH TIME THRU LOOP )--------------------- }
    UpdateScreen := FALSE;
 {--( END OF MAIN PROGRAM LOOP )---}
  UNTIL Done;


  {---------( END OF  PROGRAM )------------------------------------}

(*ResetCtrlBrk; *)                       {MUST Do this or Bad Stuff!!}
  Textmode(Co80);                        {Put the Display back to normal }
END.
